@@ -84,7 +84,7 @@ def get_cache_path(extra_name=None):
 
 
 DEFAULT_CONFIG = {
-    'max_neighbors': 10,
+    'max_neighbors': 15,  # Average length of an album in tracks + 1.
     'max_distance': 0.999,
     'history_max_pkg': 10000,
     'history_timeout': 1200,
@@ -97,8 +97,8 @@ DEFAULT_CONFIG = {
     'rebuild_stupid_threshold': 350,
     'recom_history_sieving': True,
     'recom_history_penalty': {
-        'album': 3,
-        'artist': 2
+        'album': 5,
+        'artist': 3
     }
 }
 
